
# Code overview

## Packages overview

The following packages are cloned in the workspace:

- o80 : used for interprocess communication. See [documentation](https://intelligent-soft-robots.github.io/code_documentation/o80/docs/html/index.html).

- mujoco200 : source code for mujoco

- contacts :
    - folder [ball_trajectories](https://github.com/intelligent-soft-robots/context/tree/master/python/context/trajectories) : pre-recorded ball trajectories
    - hosts of the python package "context", which has the BallTrajectory class (allows to replay pre-recorded trajectories). See [source](https://github.com/intelligent-soft-robots/context/blob/master/python/context/ball_trajectories.py)

- pam_mujoco : main package, see documentation below


## Pam Mujoco

pam_mujoco is the main package. See [source](https://github.com/intelligent-soft-robots/pam_mujoco).

For a concrete example of usage see this python [demo](https://github.com/intelligent-soft-robots/pam_mujoco/blob/master/demos/contact_examples.py).

### folders

- [models](https://github.com/intelligent-soft-robots/pam_mujoco/tree/master/models) : hosts of the xml mujoco model files

- [include](https://github.com/intelligent-soft-robots/pam_mujoco/tree/master/include/pam_mujoco) : c++ API for contact management (see [contacts.hpp](https://github.com/intelligent-soft-robots/pam_mujoco/blob/master/include/pam_mujoco/contacts.hpp) and [controllers](https://github.com/intelligent-soft-robots/pam_mujoco/blob/master/include/pam_mujoco/controllers.hpp)(see documentation below)

- [srcpy](https://github.com/intelligent-soft-robots/pam_mujoco/tree/master/srcpy) : python wrappers over c++ functionalities. Generates the "pam_mujoco" python package (see [source](https://github.com/intelligent-soft-robots/pam_mujoco/blob/master/srcpy/wrappers.cpp)). See below for details on the python API.

- [demos](https://github.com/intelligent-soft-robots/pam_mujoco/tree/master/demos) : executable demos. To run any of the demo : 'python <file_name>.py'.

### running overview

The expected usage of the API is as follow. The user code (either in c++ or in python):

- initializes mujoco (see function init_mujoco in [run_mujoco.hpp](https://github.com/intelligent-soft-robots/pam_mujoco/blob/master/include/pam_mujoco/run_mujoco.hpp)) (note: this function assumes a valid licence key mjkey.txt to be found in /opt/mujoco).

- adds a selection of controllers. A controller is a class which implements the [pam_mujoco::ControllerBase interface](https://github.com/intelligent-soft-robots/pam_mujoco/blob/master/include/pam_mujoco/controllers.hpp), i.e. it implements the function 'void apply(const mjModel* m, mjData* d)'. Functions for adding controllers can be found also in [run_mujoco.hpp](https://github.com/intelligent-soft-robots/pam_mujoco/blob/master/include/pam_mujoco/run_mujoco.hpp) (see for example add_mirror_robot).

- calls the execute function (see execute in [run_mujoco.hpp](https://github.com/intelligent-soft-robots/pam_mujoco/blob/master/include/pam_mujoco/run_mujoco.hpp)). This spawns a mujoco process, which will call at each iteration the "apply" function of each controller (sequentially, in the order they were added).

### existing controllers

- [mirror_robot](https://github.com/intelligent-soft-robots/pam_mujoco/blob/master/include/pam_mujoco/mirror_robot.hpp) : this controller instantiate a o80 backend that will compute the desired state of a robot (i.e. desired position and velocity of each joint) and overwrite mjData with the corresponding values. 

- [mirror_balls](https://github.com/intelligent-soft-robots/pam_mujoco/blob/master/include/pam_mujoco/mirror_balls.hpp) : this controller instantiate a o80 backend that will compute the desired state of balls and overwrite mjData with the corresponding values. This can be used, for example, to replay a trajectory (see [demo](https://github.com/intelligent-soft-robots/pam_mujoco/blob/master/demos/play_trajectory.py)). [run_mujoco.hpp](https://github.com/intelligent-soft-robots/pam_mujoco/blob/master/include/pam_mujoco/run_mujoco.hpp) provides currently 2 functions to add related controllers : add_mirror_one_ball (mirroring one ball) and add_mirror_until_contact_one_ball. The latest will have mujoco "mirroring" the ball (i.e. overwriting corresponding entry in mjData) until the ball enters in contact with an object, which is detected through the string "segment_id_contact" passed in argument. See "contact" controller right below.

- [contact_ball](https://github.com/intelligent-soft-robots/pam_mujoco/blob/master/include/pam_mujoco/contacts.hpp) : A 'ContactBall' controller detects contacts between a ball and another object (i.e. a racket or the table), and log corresponding information. In case of contact, it then overwrite in mjData the data corresponding to the ball in order to enforce a customized contact model (see recompute_state_after_contact function in [contacts.hpp](https://github.com/intelligent-soft-robots/pam_mujoco/blob/master/include/pam_mujoco/contacts.hpp)). At each iteration, contact information are written in the shared memory as serialized instances of ContactInformation (also declared in [contacts.hpp](https://github.com/intelligent-soft-robots/pam_mujoco/blob/master/include/pam_mujoco/contacts.hpp)). This allows other processes (e.g. see "mirror_balls" controllers right above) to access contact information (see also python [demo](https://github.com/intelligent-soft-robots/pam_mujoco/blob/master/demos/contact_examples.py)).

- [burst_controller](https://github.com/intelligent-soft-robots/pam_mujoco/blob/master/include/pam_mujoco/burst_controller.hpp) : this controller will "stop" mujoco iteration, "unblocking" it based on bursting command send by a corresponding o80 frontend. See [o80 bursting mode documentation](https://intelligent-soft-robots.github.io/code_documentation/o80/docs/html/doc/05.bursting.html) as well as the [mirror_robot demo](https://github.com/intelligent-soft-robots/pam_mujoco/blob/master/demos/mirror_robot.py) of this package.

### adding a new controller

- create a new class inhering from [pam_mujoco::ControllerBase](https://github.com/intelligent-soft-robots/pam_mujoco/blob/master/include/pam_mujoco/controllers.hpp).

- in [run_mujoco.hpp](https://github.com/intelligent-soft-robots/pam_mujoco/blob/master/include/pam_mujoco/run_mujoco.hpp) / [run_mujoco.cpp](https://github.com/intelligent-soft-robots/pam_mujoco/blob/master/src/run_mujoco.cpp) add a function that will instantiate the controller and add it to pam_mujoco::Controllers. See for example the code of "add_mirror_robot" in run_mujoco.cpp.

- register this function in [srcpy/wrappers.cpp](https://github.com/intelligent-soft-robots/pam_mujoco/blob/master/srcpy/wrappers.cpp). This will make it accessible from the pam_mujoco python package




